# Generated by Django 5.2.7 on 2025-10-14 12:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("whitebox_plugin_flight_management", "0006_alter_keymoment_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="flightsession",
            name="arrival_location",
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AddField(
            model_name="flightsession",
            name="takeoff_location",
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AddField(
            model_name="flightsession",
            name="waypoints",
            field=models.JSONField(blank=True, default=list),
        ),
    ]
