# Generated by Django 5.2.7 on 2025-10-03 21:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("whitebox_plugin_flight_management", "0005_keymoment"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="keymoment",
            options={"ordering": ["started_at"]},
        ),
        migrations.AddField(
            model_name="flightsessionrecording",
            name="ended_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="flightsessionrecording",
            name="started_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="flightsessionrecording",
            name="updated_at",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name="flightsessionrecording",
            name="status",
            field=models.IntegerField(
                choices=[
                    (10, "Created"),
                    (20, "Recording"),
                    (30, "Processing"),
                    (50, "Ready"),
                ],
                default=10,
            ),
        ),
    ]
