# Generated by Django 5.2.6 on 2025-09-22 23:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("whitebox_plugin_flight_management", "0002_flightsessionrecording"),
    ]

    operations = [
        migrations.AddField(
            model_name="flightsession",
            name="name",
            field=models.CharField(default="Unnamed Flight Session", max_length=128),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="flightsessionrecording",
            name="status",
            field=models.IntegerField(
                choices=[(10, "Not Ready"), (50, "Ready")], default=10
            ),
        ),
    ]
